ChannelLayout

Struct ChannelLayout 

pub struct ChannelLayout(/* private fields */);

Implementations

Source

impl ChannelLayout

pub const FRONT_LEFT: Self

pub const FRONT_RIGHT: Self

pub const FRONT_CENTER: Self

pub const LOW_FREQUENCY: Self

pub const BACK_LEFT: Self

pub const BACK_RIGHT: Self

pub const FRONT_LEFT_OF_CENTER: Self

pub const FRONT_RIGHT_OF_CENTER: Self

pub const BACK_CENTER: Self

pub const SIDE_LEFT: Self

pub const SIDE_RIGHT: Self

pub const TOP_CENTER: Self

pub const TOP_FRONT_LEFT: Self

pub const TOP_FRONT_CENTER: Self

pub const TOP_FRONT_RIGHT: Self

pub const TOP_BACK_LEFT: Self

pub const TOP_BACK_CENTER: Self

pub const TOP_BACK_RIGHT: Self

pub const STEREO_LEFT: Self

pub const STEREO_RIGHT: Self

pub const WIDE_LEFT: Self

pub const WIDE_RIGHT: Self

pub const SURROUND_DIRECT_LEFT: Self

pub const SURROUND_DIRECT_RIGHT: Self

pub const LOW_FREQUENCY_2: Self

pub const MONO: Self

pub const STEREO: Self

pub const _2POINT1: Self

pub const _2_1: Self

pub const SURROUND: Self

pub const _3POINT1: Self

pub const _4POINT0: Self

pub const _4POINT1: Self

pub const _2_2: Self

pub const QUAD: Self

pub const _5POINT0: Self

pub const _5POINT1: Self

pub const _5POINT0_BACK: Self

pub const _5POINT1_BACK: Self

pub const _6POINT0: Self

pub const _6POINT0_FRONT: Self

pub const HEXAGONAL: Self

pub const _6POINT1: Self

pub const _6POINT1_BACK: Self

pub const _6POINT1_FRONT: Self

pub const _7POINT0: Self

pub const _7POINT0_FRONT: Self

pub const _7POINT1: Self

pub const _7POINT1_WIDE: Self

pub const _7POINT1_WIDE_BACK: Self

pub const OCTAGONAL: Self

pub const HEXADECAGONAL: Self

pub const STEREO_DOWNMIX: Self

pub const _3POINT1POINT2: Self

pub const _5POINT1POINT2_BACK: Self

pub const _5POINT1POINT4_BACK: Self

pub const _7POINT1POINT2: Self

pub const _7POINT1POINT4_BACK: Self

pub const CUBE: Self

Source

impl ChannelLayout

pub const fn empty() -> Self

Get a flags value with all bits unset.

pub const fn all() -> Self

Get a flags value with all known bits set.

pub const fn bits(&self) -> c_ulonglong

Get the underlying bits value.

The returned value is exactly the bits set in this flags value.

Examples found in repository?
examples/transcode-audio.rs (line 21)
9fn filter(
10    spec: &str,
11    decoder: &codec::decoder::Audio,
12    encoder: &codec::encoder::Audio,
13) -> Result<filter::Graph, ffmpeg::Error> {
14    let mut filter = filter::Graph::new();
15
16    let args = format!(
17        "time_base={}:sample_rate={}:sample_fmt={}:channel_layout=0x{:x}",
18        decoder.time_base(),
19        decoder.rate(),
20        decoder.format().name(),
21        decoder.channel_layout().bits()
22    );
23
24    filter.add(&filter::find("abuffer").unwrap(), "in", &args)?;
25    filter.add(&filter::find("abuffersink").unwrap(), "out", "")?;
26
27    {
28        let mut out = filter.get("out").unwrap();
29
30        out.set_sample_format(encoder.format());
31        out.set_channel_layout(encoder.channel_layout());
32        out.set_sample_rate(encoder.rate());
33    }
34
35    filter.output("in", 0)?.input("out", 0)?.parse(spec)?;
36    filter.validate()?;
37
38    println!("{}", filter.dump());
39
40    if let Some(codec) = encoder.codec() {
41        if !codec
42            .capabilities()
43            .contains(ffmpeg::codec::capabilities::Capabilities::VARIABLE_FRAME_SIZE)
44        {
45            filter
46                .get("out")
47                .unwrap()
48                .sink()
49                .set_frame_size(encoder.frame_size());
50        }
51    }
52
53    Ok(filter)
54}
pub const fn from_bits(bits: c_ulonglong) -> Option<Self>

Convert from a bits value.

This method will return None if any unknown bits are set.

pub const fn from_bits_truncate(bits: c_ulonglong) -> Self

Convert from a bits value, unsetting any unknown bits.

pub const fn from_bits_retain(bits: c_ulonglong) -> Self

Convert from a bits value exactly.

pub fn from_name(name: &str) -> Option<Self>

Get a flags value with the bits of a flag with the given name set.

This method will return None if name is empty or doesn’t correspond to any named flag.

pub const fn is_empty(&self) -> bool

Whether all bits in this flags value are unset.

pub const fn is_all(&self) -> bool

Whether all known bits in this flags value are set.

pub const fn intersects(&self, other: Self) -> bool

Whether any set bits in a source flags value are also set in a target flags value.

pub const fn contains(&self, other: Self) -> bool

Whether all set bits in a source flags value are also set in a target flags value.

pub fn insert(&mut self, other: Self)

The bitwise or (|) of the bits in two flags values.

pub fn remove(&mut self, other: Self)

The intersection of a source flags value with the complement of a target flags value (&!).

This method is not equivalent to self & !other when other has unknown bits set. remove won’t truncate other, but the ! operator will.

pub fn toggle(&mut self, other: Self)

The bitwise exclusive-or (^) of the bits in two flags values.

pub fn set(&mut self, other: Self, value: bool)

Call insert when value is true or remove when value is false.

pub const fn intersection(self, other: Self) -> Self

The bitwise and (&) of the bits in two flags values.

pub const fn union(self, other: Self) -> Self

The bitwise or (|) of the bits in two flags values.

pub const fn difference(self, other: Self) -> Self

The intersection of a source flags value with the complement of a target flags value (&!).

This method is not equivalent to self & !other when other has unknown bits set. difference won’t truncate other, but the ! operator will.

pub const fn symmetric_difference(self, other: Self) -> Self

The bitwise exclusive-or (^) of the bits in two flags values.

pub const fn complement(self) -> Self

The bitwise negation (!) of the bits in a flags value, truncating the result.

Source

impl ChannelLayout

pub const fn iter(&self) -> Iter<ChannelLayout>

Yield a set of contained flags values.

Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.

pub const fn iter_names(&self) -> IterNames<ChannelLayout>

Yield a set of contained named flags values.

This method is like iter, except only yields bits in contained named flags. Any unknown bits, or bits not corresponding to a contained flag will not be yielded.

Source

impl ChannelLayout

pub fn channels(&self) -> i32

Examples found in repository?
examples/transcode-audio.rs (line 93)
65fn transcoder<P: AsRef<Path> + ?Sized>(
66    ictx: &mut format::context::Input,
67    octx: &mut format::context::Output,
68    path: &P,
69    filter_spec: &str,
70) -> Result<Transcoder, ffmpeg::Error> {
71    let input = ictx
72        .streams()
73        .best(media::Type::Audio)
74        .expect("could not find best audio stream");
75    let context = ffmpeg::codec::context::Context::from_parameters(input.parameters())?;
76    let mut decoder = context.decoder().audio()?;
77    let codec = ffmpeg::encoder::find(octx.format().codec(path, media::Type::Audio))
78        .expect("failed to find encoder")
79        .audio()?;
80    let global = octx
81        .format()
82        .flags()
83        .contains(ffmpeg::format::flag::Flags::GLOBAL_HEADER);
84
85    decoder.set_parameters(input.parameters())?;
86
87    let mut output = octx.add_stream(codec)?;
88    let context = ffmpeg::codec::context::Context::from_parameters(output.parameters())?;
89    let mut encoder = context.encoder().audio()?;
90
91    let channel_layout = codec
92        .channel_layouts()
93        .map(|cls| cls.best(decoder.channel_layout().channels()))
94        .unwrap_or(ffmpeg::channel_layout::ChannelLayout::STEREO);
95
96    if global {
97        encoder.set_flags(ffmpeg::codec::flag::Flags::GLOBAL_HEADER);
98    }
99
100    encoder.set_rate(decoder.rate() as i32);
101    encoder.set_channel_layout(channel_layout);
102    #[cfg(not(feature = "ffmpeg_7_0"))]
103    {
104        encoder.set_channels(channel_layout.channels());
105    }
106    encoder.set_format(
107        codec
108            .formats()
109            .expect("unknown supported formats")
110            .next()
111            .unwrap(),
112    );
113    encoder.set_bit_rate(decoder.bit_rate());
114    encoder.set_max_bit_rate(decoder.max_bit_rate());
115
116    encoder.set_time_base((1, decoder.rate() as i32));
117    output.set_time_base((1, decoder.rate() as i32));
118
119    let encoder = encoder.open_as(codec)?;
120    output.set_parameters(&encoder);
121
122    let filter = filter(filter_spec, &decoder, &encoder)?;
123
124    let in_time_base = decoder.time_base();
125    let out_time_base = output.time_base();
126
127    Ok(Transcoder {
128        stream: input.index(),
129        filter,
130        decoder,
131        encoder,
132        in_time_base,
133        out_time_base,
134    })
135}
pub fn default(number: i32) -> ChannelLayout